{************************************************************************
*									*
*  JUMP -- JUMP_PERSONA.PAS						*
*									*
*  JUMP_PERSONA implements the PERSONA system services to change	*
*  process characteristics.						*
*									*
*************************************************************************
*  Author and Maintainer: Jonathan Ridler.				*
*									*
*  This software is owned and maintained privately by the author,	*
*  Jonathan Ridler. It is NOT in any way produced, owned, supported,	*
*  maintained or endorsed by Hewlett Packard Pty Ltd or any of its	*
*  legal entities.							*
*									*
*  Email: vmsjump@internode.on.net					*
*************************************************************************
*************************************************************************
***								      ***
***			    LICENCE NOTICE			      ***
***								      ***
***  This software is COPYRIGHT (c) 1993-2022 Jonathan Ridler.	      ***
***  ALL RIGHTS RESERVED.					      ***
***								      ***
***  Please READ the file JUMP_LICENCE.TXT which contains the	      ***
***  complete Licence and all conditions of use for JUMP.	      ***
***								      ***
***  NOTE: By using JUMP or any part of the JUMP distribution for     ***
***	   any purpose, you are agreeing to all conditions of the     ***
***	   JUMP licence.					      ***
***								      ***
*************************************************************************
*************************************************************************
*									*
*  History:								*
*	01-Feb-2012	JER	Original version for JUMP V6.0. 	*
*	03-Feb-2012	JER	JUMP V6.1.				*
*	21-Feb-2012	JER	JUMP V6.1a; added build level ident;	*
*				Fixed debug structure definition.	*
*	28-Aug-2012	JER	A little more info in the debug output. *
************************************************************************}

[INHERIT ('SYS$LIBRARY:STARLET',
	  'SYS$LIBRARY:PASCAL$LIB_ROUTINES')]

MODULE Jump_Persona (OUTPUT) ;

CONST

{ The effective length of a username is 12 characters. However, the internal
  fields are 32 characters.  Use 12 characters as the maximum length.

  ENSURE the constant is identical in the JUMP, JUMP_POKER and JUMP_PERSONA
  modules!  }

Max_Username_Len = 12 ;

TYPE

$UWORD = [WORD] 0..65535 ;
$UQUAD = [QUAD,UNSAFE] RECORD  L0 , L1 : UNSIGNED ;  END ;

Item_List_Cell = RECORD
		   CASE INTEGER OF
		     1: (			{ Normal Cell }
			 Buffer_Length : $UWORD ;
			 Item_Code     : $UWORD ;
			 Buffer_Addr   : UNSIGNED ;
			 Return_Addr   : UNSIGNED
			) ;
		     2: (			{ Terminator }
			 Terminator    : UNSIGNED
			) ;
		 END ;

Item_List_Template (Count:INTEGER) = ARRAY [1..Count] OF Item_List_Cell ;

Debug_Mask = RECORD
	       CASE INTEGER OF
	       1: (All_Bits : UNSIGNED ;) ;    { 32 bits - roomy! :) }
	       2: (Info     : [BIT, POS(0)]  BOOLEAN ;	   { General Info }
		   D1	    : [BIT, POS(1)]  BOOLEAN ;	   { Diagnostic info }
		   D2	    : [BIT, POS(2)]  BOOLEAN ;	   { Diagnostic info }
		   D3	    : [BIT, POS(3)]  BOOLEAN ;	   { Diagnostic info }
		   D4	    : [BIT, POS(4)]  BOOLEAN ;	   { Diagnostic info }
		   D5	    : [BIT, POS(5)]  BOOLEAN ;	   { Diagnostic info }
		   D6	    : [BIT, POS(6)]  BOOLEAN ;	   { Diagnostic info }
		   D7	    : [BIT, POS(7)]  BOOLEAN ;	   { Diagnostic info }
		   D8	    : [BIT, POS(8)]  BOOLEAN ;	   { Diagnostic info }
		   D9	    : [BIT, POS(9)]  BOOLEAN ;	   { Diagnostic info }
		   X1	    : [BIT, POS(11)] BOOLEAN ;	   { Change execution }
		   X2	    : [BIT, POS(12)] BOOLEAN ;	   { Change execution }
		   X3	    : [BIT, POS(13)] BOOLEAN ;	   { Change execution }
		   X4	    : [BIT, POS(14)] BOOLEAN ;	   { Change execution }
		   X5	    : [BIT, POS(15)] BOOLEAN ;	   { Change execution }
		   X6	    : [BIT, POS(16)] BOOLEAN ;	   { Change execution }
		   X7	    : [BIT, POS(17)] BOOLEAN ;	   { Change execution }
		   X8	    : [BIT, POS(18)] BOOLEAN ;	   { Change execution }
		   X9	    : [BIT, POS(19)] BOOLEAN ;) ;  { Change execution }
	     END ;

Privilege = (Cmkrnl,	{  0: May change Mode to Kernel }
	     Cmexec,	{  1: May change Mode to Exec; MUST follow Cmkrnl }
	     Sysnam,	{  2: May insert in system logical name table }
	     Grpnam,	{  3: May insert in group l.n.tab; MUST follow Sysnam }
	     Allspool,	{  4: May allocate spooled device }
	     Detach,	{  5: May create detached processes (aka Impersonate) }
	     Diagnose,	{  6: May diagnose devices }
	     Log_Io,	{  7: May do logical I/O }
	     Group,	{  8: May affect other processes in same UIC group }
	     Acnt,	{  9: May suppress accounting }
	     Prmceb,	{ 10: May create permanent common event clusters }
	     Prmmbx,	{ 11: May create permanent mail box }
	     Pswapm,	{ 12: May change process mode }
	     Altpri,	{ 13: May set any priority value }
	     Setprv,	{ 14: May set any privilege bits }
	     Tmpmbx,	{ 15: May create temporary mailbox }
	     World,	{ 16: May affect other processes in the world }
	     Mount,	{ 17: May execute mount ACP functions }
	     Oper,	{ 18: Operator privilege }
	     Exquota,	{ 19: May exceed disk quotas }
	     Netmbx,	{ 20: May create network device }
	     Volpro,	{ 21: May override volume protection }
	     Phy_Io,	{ 22: May do physical I/O }
	     Bugchk,	{ 23: May make bug check error log entries }
	     Prmgbl,	{ 24: May create permanent global sections }
	     Sysgbl,	{ 25: May create system-wide global sections }
	     Pfnmap,	{ 26: May map to section by PFN }
	     Shmem,	{ 27: May allocate structures in shared memory }
	     Sysprv,	{ 28: Eligible for system protection field }
	     Bypass,	{ 29: May bypass UIC-based protection }
	     Syslck,	{ 30: May create system-wide locks }
	     Share,	{ 31: May assign channel to non-shared device }
	     Upgrade,	{ 32: May upgrade object integrity }
	     Downgrade, { 33: May downgrade object secrecy }
	     Grpprv,	{ 34: Group access via system protection field }
	     Readall,	{ 35: May read any object }
	     Import,	{ 36: May set classification for unlabeled object }
	     Audit,	{ 37: May direct audit to system security audit log }
	     Security,	{ 38: May perform security functions }
	     Pbit39, Pbit40, Pbit41, Pbit42, Pbit43, Pbit44, Pbit45,
	     Pbit46, Pbit47, Pbit48, Pbit49, Pbit50, Pbit51, Pbit52,
	     Pbit53, Pbit54, Pbit55, Pbit56, Pbit57, Pbit58, Pbit59,
	     Pbit60, Pbit61, Pbit62, Pbit63) ;	{ 39-63: dummy bits: 2 LONGs }

Privset = PACKED SET OF Privilege ;		{ Allow easy bit union, etc. }

Status_Block_Type = [UNSAFE] PACKED ARRAY [1..4] OF $UWORD ;

Just_A_String = VARYING [1024] OF CHAR VALUE ZERO ;

VAR

{************************************************************************
 ****  INCREMENT the BUILD LEVEL every time the module is changed!!  ****
 ************************************************************************}

Persona_Bl : [GLOBAL] VARYING [12] OF CHAR := '1.010' ;

Acme_Agent_Id : [STATIC,VOLATILE] UNSIGNED := 0 ;

Acme_Agent_Name ,
Orig_User : [EXTERNAL] VARYING [Max_Username_Len] OF CHAR ;

Bugger : [EXTERNAL] Debug_Mask ;	{ Debug flags }

Dbgfile : [EXTERNAL] TEXT ;	{ Debug output file }

Privs : Privset := [Cmkrnl,Cmexec,Sysnam,Detach,Tmpmbx,Netmbx,Sysprv] ;


PROCEDURE Zip (Rst1 : UNSIGNED ;
	       Rst2 : UNSIGNED := SS$_NORMAL) ; EXTERNAL ;


FUNCTION Str_Privs (Privs : Privset) : Just_A_String ; EXTERNAL ;

PROCEDURE Get_Priv_Info ;

{ Get relevant information about the invoker of the program. }

  VAR	Rst : INTEGER := 0 ;
	Iosb : Status_Block_Type := ZERO ;
	Item_List : Item_List_Template (4) := ZERO ;

	Image_Work_Priv ,		{ Image's active  privs }
	Image_Perm_Priv ,		{ Image's default privs }
	Image_Auth_Priv 		{ Image's auth'd  privs }
	    : [UNSAFE,VOLATILE]  Privset := ZERO ;

  BEGIN 	{ Get_Priv_Info }
  Item_List[1].Buffer_Length := 8 ;
  Item_List[1].Item_Code     := JPI$_IMAGE_PERMPRIV ;
  Item_List[1].Buffer_Addr   := IADDRESS (Image_Perm_Priv) ;
  Item_List[1].Return_Addr   := 0 ;

  Item_List[2].Buffer_Length := 8 ;
  Item_List[2].Item_Code     := JPI$_IMAGE_AUTHPRIV ;
  Item_List[2].Buffer_Addr   := IADDRESS (Image_Auth_Priv) ;
  Item_List[2].Return_Addr   := 0 ;

  Item_List[3].Buffer_Length := 8 ;
  Item_List[3].Item_Code     := JPI$_IMAGE_WORKPRIV ;
  Item_List[3].Buffer_Addr   := IADDRESS (Image_Work_Priv) ;
  Item_List[3].Return_Addr   := 0 ;

  Item_List[4].Terminator    := 0 ;    { Terminate the item list }

  Rst := $GETJPIW (Itmlst := Item_List,
		   Iosb   := Iosb) ;
  Zip (Rst,Iosb[1]) ;

  WRITELN (Dbgfile,'*** Image perm privs:',Str_Privs(Image_Perm_Priv)) ;
  WRITELN (Dbgfile,'*** Image auth privs:',Str_Privs(Image_Auth_Priv)) ;
  WRITELN (Dbgfile,'*** Image work privs:',Str_Privs(Image_Work_Priv)) ;
  END ; 	{ of Get_Priv_Info }


PROCEDURE Check_Persona_Extension ;

  VAR  Rst : UNSIGNED := 0 ;
       Checked : [STATIC] BOOLEAN := FALSE ;

  { Check to see if the requested external authentication extension
    is available. }

  BEGIN 	{ Check_Persona_Extension }
  IF NOT Checked  THEN
     BEGIN
     Rst := $PERSONA_EXTENSION_LOOKUP (Acme_Agent_Name,Acme_Agent_Id) ;
     IF Bugger.D4  THEN
	WRITELN (Dbgfile,'*** Check_EID: Rst = ',Rst:1,' / Name = ',
		 Acme_Agent_Name,' / EID = ',Acme_Agent_Id:1) ;
     IF Rst = SS$_NOSUCHEXT  THEN
	BEGIN
	LIB$SIGNAL (SS$_NOEXTAUTH-4) ;	   { WARN: Ext Auth not supported }
	WRITELN ;
	END
      ELSE
     IF Acme_Agent_Name = 'VMS'  THEN
	Rst := ACME$_FAILURE	   { Ext Auth already supported -- ignore }
      ELSE
	Zip (Rst) ;
     Checked := TRUE ;
     END ;
  END ; 	{ Check_Persona_Extension }


[GLOBAL] PROCEDURE Change_Persona (New_Username : VARYING [Len] OF CHAR) ;

  VAR  Rst ,
       New_Persona : UNSIGNED := 0 ;
       Item_List : Item_List_Template (2) := ZERO ;

  BEGIN 	{ Change_Persona }

  IF Bugger.D4	THEN
     WRITELN (Dbgfile,'*** Change_Persona from ',
		      Orig_User,' to ',New_Username) ;

  Rst := $PERSONA_CREATE (Persona := New_Persona,
			  Usrnam  := New_Username) ;
  Zip (Rst) ;

  IF Bugger.D4	THEN
     WRITELN (Dbgfile,'*** Persona Created! ',Rst:1) ;

  Check_Persona_Extension ;

  { Now modify the privs and then assume the new persona. }

  IF Bugger.D4	THEN  Get_Priv_Info ;

  Item_List[1].Buffer_Length := 8 ;
  Item_List[1].Item_Code     := ISS$_IMAGE_WORKPRIV ;
  Item_List[1].Buffer_Addr   := IADDRESS (Privs) ;
  Item_List[1].Return_Addr   := 0 ;

  Item_List[2].Terminator    := 0 ;	{ Terminate the item list }

  Rst := $PERSONA_MODIFY (New_Persona,Item_List) ;
  Zip (Rst) ;

  IF Bugger.D4	THEN
     WRITELN (Dbgfile,'*** Persona Modified! ',Rst:1) ;

  Rst := $PERSONA_ASSUME (New_Persona) ;
  Zip (Rst) ;

  IF Bugger.D4	THEN
     WRITELN (Dbgfile,'*** Persona Assumed! ',Rst:1) ;
  END ; 	{ Change_Persona }

END.
